@@ -30,7 +30,7 @@ void sock_shutdown(void);
 
 int sock_error(void);
 
-sock_t sock_connect(const char * const host, const unsigned int port);
+sock_t sock_connect(const char * const host, const unsigned short port);
 int sock_close(const sock_t sock);
 
 int sock_set_blocking(const sock_t sock);
@@ -40,9 +40,6 @@ int sock_write(const sock_t sock, const void * const buff, const size_t len);
 int sock_is_recoverable(const int error);
 /* checks for an error after connect, return 0 if connect successful */
 int sock_connect_error(const sock_t sock);
-
-int sock_srv_lookup(const char *service, const char *proto,
-		     const char *domain, char *resulttarget,
-		     int resulttargetlength, int *resultport);
+int sock_set_keepalive(const sock_t sock, int timeout, int interval);
 
 #endif /* __LIBSTROPHE_SOCK_H__ */
